#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::Hash::Passwords;

=head1 NAME

SrSv::Hash::Passwords - Handle passwords, hashing, and verifying the hashes.

=cut

use strict;
use SrSv::Hash::SaltedHash;
use SrSv::Conf qw( main );

use Exporter 'import';
BEGIN { 
	my %constants = (
		SrSv_HASH => 'SHA256',
		# Can't make this 8 unless we have 64-bit native ints
		# give it a couple years until 64bit is normal.
		# It will likely require a way to tell in the db.
		# maybe {SSHA256-8}
		SrSv_SALTLENGTH => 4,
	);

	require constant; import constant (\%constants);
	our @EXPORT = qw(hash_pass validate_pass is_hashed keys(%constants));
}

=head2 

  hash_pass($pass)
  	If hashed-passwords is enabled in main.conf, returns a hashed password in a string.
	Otherwise returns $pass unmodified.

=cut

sub hash_pass($) {
	my ($pass) = @_;
	if($main_conf{'hashed-passwords'}) {
		my $hashobj = SrSv::Hash::SaltedHash->new( { algorithm => SrSv_HASH, salt_len => SrSv_SALTLENGTH } );
		$hashobj->add($pass);
		return $hashobj->generate;
	}
	else {
		return $pass;
	}
}

=head2

  validate_pass($hashedPass, $pass)
	Decodes the hashedPass.
	- If $hashedPass is a valid SSHA256 hash-string, it and determines whether $pass matches $hashedPass
	- If $hashedPass is not a valid SSHA256 hash-string, it returns ($hashedPass eq $pass)

=cut
sub validate_pass($$) {
	my ($hashedPass, $pass) = @_;
	if (my $hash_type = is_hashed($hashedPass)) {
		my $hashobj = SrSv::Hash::SaltedHash->new( { algorithm => SrSv_HASH, salt_len => SrSv_SALTLENGTH } );
		return $hashobj->validate($hashedPass, $pass);
	} else {
		return $hashedPass eq $pass;
	}
}

sub is_hashed($) {
	if ($_[0] =~ /^\{S(.*)\}/) {
		return $1;
	} else {
		return undef;
	}
}

1;
